import base64
import logging
import os

from starlette.applications import Starlette
from starlette.middleware import Middleware
from starlette.middleware.sessions import SessionMiddleware
from starlette.requests import Request
from starlette.responses import PlainTextResponse, RedirectResponse, Response
from starlette.routing import Mount, Route
from starlette.staticfiles import StaticFiles
from starlette_wtf import CSRFProtectMiddleware

from vexillum import cryptography
from vexillum.flags import FLAGS
from vexillum.middleware import AuthorizationMiddleware
from vexillum.register import register
from vexillum.templates import templates
from vexillum.util import package_dir

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


async def homepage(request: Request) -> Response:
    return templates.TemplateResponse(request, "index.html", {"profile": request.scope["profile"], "flags": FLAGS})


async def logout(request: Request) -> Response:
    return cryptography.delete_profile(RedirectResponse("/sumo", status_code=303))


def fformat(request: Request, flag: bytes) -> Response:
    return templates.TemplateResponse(request, "flag.html", {"flag": base64.b64encode(flag)})


async def buy_flag(request: Request) -> Response:
    flag_id = int(request.path_params["flag_id"])
    flag = FLAGS.get(flag_id)

    if flag is None:
        return PlainTextResponse("Flag not found", status_code=404)

    profile: cryptography.Profile = request.scope["profile"]

    if profile.balance < flag.price:
        return PlainTextResponse("Not enough money", status_code=403)
    
    profile = cryptography.Profile(profile.name, profile.balance - flag.price)

    return cryptography.encode_profile(templates.TemplateResponse(request, "flag.html", {"flag": flag, "motto": await flag.produce(request)}), profile)


app = Starlette(
    debug=bool(os.environ.get("DEBUG")),
    routes=[
        Mount("/static", app=StaticFiles(directory=package_dir / "static"), name="static"),
        Route("/subsigno", register, methods=["GET", "POST"], name="register", middleware=[
            Middleware(SessionMiddleware, secret_key=os.environ["SECRET"]),
            Middleware(CSRFProtectMiddleware, csrf_secret=os.environ["SECRET"])
        ]),
        Mount(
            "/",
            routes=[
                Route("/", homepage),
                Route("/exire", logout),
                Route("/sumo/{flag_id:int}", buy_flag, methods=["POST"])
            ],
            middleware=[
                Middleware(AuthorizationMiddleware)
            ],
            name="website"
        )
    ]
)


if __name__ == "__main__":
    import uvicorn
    uvicorn.run("vexillum.app:app", reload=True)
